﻿using System;
using System.Resources; 
using System.Data;
using System.Configuration;
using System.Collections;
using System.Linq;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Xml.Linq;
using System.Collections.Specialized;
using System.Reflection;
using System.Text;
using VeteransAffairs.Registries.BusinessManager;

/// <summary>
/// Summary description for BasePage
/// </summary>
public abstract class BasePage : Page
{
    #region Constructor

    protected static ResourceManager resources;

    private UserAccountManager _currentUser;
    private int _displayMode;
    private string _currentRegistry;
    private bool _errorOnSavedReceived;

    /// <summary>
    /// Set the EntityInstitution if page should check permissions based on institution.
    /// </summary>
    protected int? EntityInstitution;

    public Menu MenuHorizontalMenu;
    public Menu MenuLeftNavigation;
    public Label LabelBreadCrumb;

    public BasePage() 
    {
        resources = new ResourceManager("Portal.strings", Assembly.GetExecutingAssembly());
        this.Load += new EventHandler(BasePage_Load);
        SuppressSaveMessage = true;
    }

    public BasePage(bool errorOnSavedReceived)
    {
        _errorOnSavedReceived = errorOnSavedReceived;
        SuppressSaveMessage = true;
    }

    #endregion Constructor

    #region Properities

    public UserAccountManager CurrentUser
    {
        get
        {
            return _currentUser;
        }
    }
    public string CurrentRegistry
    {
        get
        {
            return _currentRegistry;
        }
    }
    public bool IgnoreCapabilityCheck { get; set; }
    public bool ReadOnly { get; set; }
    public bool AllowInsert { get; set; }
    public bool AllowDelete { get; set; }
    public bool AllowUpdate { get; set; }
    public bool BusinessObjectSaveEventReceived { get; set; }
    public bool SuppressSaveMessage { get; set; }
    public bool IsSaveSuccess { get; set; }

    #endregion Properties

    /// <summary>
    /// Get whether or not the page requires authentication to access.  Assume true but allow specific pages to override.
    /// </summary>
    protected virtual bool RequiresAuthentication
    {
        get { return true; }
    }
    private void BasePage_Load(object sender, EventArgs e)
    {
        // Kick the user out if the page requires authentication but the user somehow isn't (like if the auth system has been
        // maliciously bypassed).
        if (RequiresAuthentication && (!User.Identity.IsAuthenticated))
        {
            Response.Redirect("ErrorPage.aspx?id=404");
        }

        Response.Cache.SetExpires(DateTime.Now);

    }
 
    //private SessionFacade _SessionFacade = new SessionFacade();

    #region Abstract Methods

    /// <summary>
    /// TopMenuName is the menu item in the top horizontal menu where this page falls under
    /// </summary>
    public abstract string TopMenuName();

    /// <summary>
    /// MenuItemName is the value of the left menu link for this page
    /// </summary>
    public abstract string MenuItemName();

    /// <summary>
    /// The list of capabilities that a user needs to access this page.  A single page may have more than one
    /// capability associated with it.
    /// </summary>
    public abstract string AppObjectCode();
    
    #endregion Abstract Methods

    #region Public Methods

    /// <summary>
    /// Select returns the base application path.
    /// </summary>
    /// <param name="context">Context object</param>
    /// <returns>Returns the base application path.</returns>
    public static string RootPath(HttpContext context)
    {
        string urlSuffix = context.Request.Url.Authority + context.Request.ApplicationPath;
        return context.Request.Url.Scheme + @"://" + urlSuffix + "/";
    }

    public static NameValueCollection DecryptQueryString(string queryString)
    {
        //return StringHelpers.DecryptQueryString(queryString);
        NameValueCollection myColl = new NameValueCollection();
        myColl.Add("a", "1");
        return myColl;
    }

    public static string EncryptQueryString(NameValueCollection queryString)
    {        
        //return StringHelpers.EncryptQueryString(queryString);
        string myString = "";
        return myString;
    }

    /// <summary>
    /// You must pass in a string that uses the QueryStringHelper.DELIMITER as the delimiter.
    /// This will also append the "?" to the beginning of the query string.
    /// </summary>
    /// <param name="queryString"></param>
    /// <returns></returns>
    public static string EncryptQueryString(string queryString)
    {
        //return StringHelpers.EncryptQueryString(queryString);
        string qs = "";
        return qs;
    }

    //Chapter 9: Printing
    public string GetPrintButtonScript(Button btn)
    {        
        StringBuilder printButtonScript = new StringBuilder();

        //Get the postback script.
        string postback = this.Page.ClientScript.GetPostBackEventReference(btn, "");

        //Change target to a new window.  Name the window the current date and time so multiple windows can
        //be opened.
        printButtonScript.Append("var today = new Date();");
        printButtonScript.Append("var newWindowName = today.getFullYear().toString() + today.getMonth().toString() + today.getDate().toString() + today.getHours().toString() + today.getHours().toString() + today.getMinutes().toString() + today.getSeconds().toString() + today.getMilliseconds().toString();");

        printButtonScript.Append("document.forms[0].target = newWindowName;");

        //TODO: Added root path after this was turned in.
        //Show the please wait screen.
        printButtonScript.Append("window.open('" + RootPath(base.Context) + "/Reports/PleaseWait.html', newWindowName, 'scrollbars=yes,status=no,resizable=yes');");

        //Add the postback script.
        printButtonScript.Append(postback + ";");

        //Reset target back to itself so other controls will post back to this form.
        printButtonScript.Append("document.forms[0].target='_self';");
        //Return false to stop page submitting.
        printButtonScript.Append("return false;" + Environment.NewLine);

        return printButtonScript.ToString();
    }

    #endregion Public Methods

    #region Export Grid to Excel Methods

    
    public void ExportGridViewToExcel(GridView GridView1, string filename)
    {
        PrepareGridViewForExport(GridView1);

        HtmlForm form = new HtmlForm();

        string attachment = "attachment; filename=" + filename;

        Response.ClearContent();
        Response.AddHeader("content-disposition", attachment);
        Response.ContentType = "application/vnd.ms-excel";
        System.IO.StringWriter sw = new System.IO.StringWriter();
        HtmlTextWriter htw = new HtmlTextWriter(sw);

        form.Controls.Add(GridView1);

        this.Controls.Add(form);

        form.RenderControl(htw);

        //GridView1.RenderControl(htw);
        Response.Write(sw.ToString());
        Response.End();

    }

    public void PrepareGridViewForExport(Control gv)
    {

        Hashtable htControls = new Hashtable();

        htControls.Add("LinkButton", "Text");
        htControls.Add("HyperLink", "Text");
        htControls.Add("DropDownList", "SelectedItem");
        htControls.Add("CheckBox", "Checked");

        Literal l = new Literal();

        for (int i = 0; i < gv.Controls.Count; i++)
        {

            if ((null != htControls[gv.Controls[i].GetType().Name]) || (null != htControls[gv.Controls[i].GetType

               ().BaseType.Name]))
            {

                l.Text = GetControlPropertyValue(gv.Controls[i], htControls);

                gv.Controls.Remove(gv.Controls[i]);

                gv.Controls.AddAt(i, l);

            }

            if (gv.Controls[i].HasControls())
            {

                PrepareGridViewForExport(gv.Controls[i]);

            }

        }

    }

    private string GetControlPropertyValue(Control control, Hashtable htControls)
    {

        Type controlType = control.GetType();

        string strControlType = controlType.Name;

        string strReturn = "Error";

        bool bReturn;



        PropertyInfo[] ctrlProps = controlType.GetProperties();

        string ExcelPropertyName = (string)htControls[strControlType];

        if (ExcelPropertyName == null)
        {

            ExcelPropertyName = (string)htControls[control.GetType().BaseType.Name];

            if (ExcelPropertyName == null)

                return strReturn;

        }

        foreach (PropertyInfo ctrlProp in ctrlProps)
        {

            if (ctrlProp.Name == ExcelPropertyName &&

            ctrlProp.PropertyType == typeof(String))
            {

                try
                {

                    strReturn = (string)ctrlProp.GetValue(control, null);

                    break;

                }

                catch
                {

                    strReturn = "";

                }

            }

            if (ctrlProp.Name == ExcelPropertyName &&

            ctrlProp.PropertyType == typeof(bool))
            {

                try
                {

                    bReturn = (bool)ctrlProp.GetValue(control, null);

                    strReturn = bReturn ? "True" : "False";

                    break;

                }

                catch
                {

                    strReturn = "Error";

                }

            }

            if (ctrlProp.Name == ExcelPropertyName &&

            ctrlProp.PropertyType == typeof(ListItem))
            {

                try
                {

                    strReturn = ((ListItem)(ctrlProp.GetValue(control, null))).Text;

                    break;

                }

                catch
                {

                    strReturn = "";

                }

            }

        }

        return strReturn;

    }

    #endregion 

    #region Overrides

    protected override void OnPreInit(EventArgs e)
    {
        _currentRegistry = Helpers.GetConfigEntry("Registry");


        //get theme from config file
        this.Theme = CurrentRegistry; // "DVEIR";        

    }

    protected override void OnInit(EventArgs e)
    {
        base.OnInit(e);

        CheckCapabilities();

        if (!isQueryStringValid())
        {
            //redirect to error handling page
            Response.Redirect("ErrorPage.aspx?id=qs");
        }

        LabelBreadCrumb = (Label)Master.FindControl("LabelBreadCrumb");

        if (!IsPostBack)
        {
            if (!Page.MasterPageFile.Contains("RegistryNoLeftMenu"))
            {
                BuildMenusAndBreadCrumb();
            }
        }

    }

    protected void BuildMenusAndBreadCrumb()
    {
        //Get menu list based on user access
        string[] pagesAccessibleByUser = ((UserAccountManager)HttpContext.Current.User).GetPagesByCode();

        MenuHorizontalMenu = (Menu)Master.FindControl("MenuHorizontalMenu");
        MenuLeftNavigation = (Menu)Master.FindControl("MenuLeftNavigation");
        //LabelBreadCrumb = (Label)Master.FindControl("LabelBreadCrumb");

        //Populate left menu and breadcrumb - based on user roles
        MenuBuilder.BuildDynamicNavigationItems(HttpContext.Current.Request.Url.PathAndQuery, MenuHorizontalMenu, TopMenuName(), MenuLeftNavigation, MenuItemName(), LabelBreadCrumb, pagesAccessibleByUser);

    }

    protected override void OnPreRender(EventArgs e)
    {        
        //if the page has EntityInstituion then check the permissions based on institution 
        if (EntityInstitution.HasValue && EntityInstitution != 0)
        {
            CheckCapabilities(EntityInstitution.GetValueOrDefault());
        }

        base.OnPreRender(e);
        //Helpers.SetInputControlsHighlight(this, "highlight");

        SetStandardControlProperties(this.Controls);

        if (!SuppressSaveMessage)
        {
            RenderStartUpMessage(IsSaveSuccess);
        }
    }

    #endregion Overrides

    #region Virtual Methods

    public virtual void CheckCapabilities()
    {
        if (IgnoreCapabilityCheck == false)
        {            
            //Get user permissions
            _currentUser = (UserAccountManager)HttpContext.Current.User;
            if (_currentUser.HasAccessToRegistry() == false)
                NoAccessToPage();
            else
            {
                Permissions Perm = _currentUser.GetPermissions(AppObjectCode());
                if (Perm.NoAccess == true)
                    NoAccessToPage();
                else if (Perm.ReadOnly == true)
                    MakeFormReadOnly(AppObjectCode(), this.Controls);

                AllowDelete = Perm.DeleteAllowed;
                AllowInsert = Perm.InsertAllowed;
                ReadOnly = Perm.ReadOnly;
                AllowUpdate = Perm.UpdateAllowed;

            }
        }
    }

    public virtual void CheckCapabilities(int InstitutionId)
    {
        if (IgnoreCapabilityCheck == false)
        {
            //Get user permissions
            _currentUser = (UserAccountManager)HttpContext.Current.User;
            if (_currentUser.HasAccessToRegistry() == false)
                NoAccessToPage();
            else
            {
                Permissions Perm = _currentUser.GetPermissions(AppObjectCode(),InstitutionId);
                if (Perm.NoAccess == true)
                    NoAccessToPage();
                else if (Perm.ReadOnly == true)
                    MakeFormReadOnly(AppObjectCode(), this.Controls);

                AllowDelete = Perm.DeleteAllowed;
                AllowInsert = Perm.InsertAllowed;
                ReadOnly = Perm.ReadOnly;
                AllowUpdate = Perm.UpdateAllowed;

            }
        }
    }
        
    protected virtual void NoAccessToPage()
    {     
        //You do not access to the system.
        //If a page has more than one capability you should override this method because a user could
        //have access to one section but not another so you do not want them to get an error
        Response.Redirect("ErrorPage.aspx?id=100");
        //throw new AccessViolationException("You do not have access to this screen.");
    }

    public virtual void CustomReadOnlyLogic(string capabilityName)
    {
        //Override this method in a page that has custom logic for non standard controls on the screen.
    }

    /// <summary>
    /// The default implementation will make all controls disabled.
    /// If you have more than one capability associated with a page you should override this method
    /// with the special logic for each capability in the page.
    /// </summary>
    /// <param name="capabilityName"></param>
    /// <param name="controls"></param>
    public virtual void MakeFormReadOnly(string capabilityName, ControlCollection controls)
    {
        ReadOnly = true;

        MakeControlsReadOnly(controls);

        CustomReadOnlyLogic(capabilityName);
    }

    #endregion Virtual Methods

    #region Private Methods
    
    private void RenderStartUpMessage(bool IsSaveSuccessMessage)
    {
        StringBuilder sb = new StringBuilder();
        string Message = CreateStartUpMessage(IsSaveSuccessMessage);

        if (Message != string.Empty)
        {
            HiddenField HiddenFieldShowSaveMessage = this.Master.FindControl("HiddenFieldShowSaveMessage") as HiddenField;
            HiddenFieldShowSaveMessage.Value = "1";
            HiddenField HiddenFieldSaveMessage = this.Master.FindControl("HiddenFieldSaveMessage") as HiddenField;
            HiddenFieldSaveMessage.Value = Message;
        }
    }
    private string CreateStartUpMessage(bool isSaveSuccessMessage)
    {
        string Message = string.Empty;

        if ( !SuppressSaveMessage)
            if (isSaveSuccessMessage)
            {
                Message = "Save Successful.";
            }
            else
            {
                Message = "Save Failed.";
            }
        return Message;
    }
    private void MakeControlsReadOnly(ControlCollection controls)
    {                        
        foreach (Control c in controls)
        {
            // Don't set to readonly if control overrides page's ReadOnly status.
            BaseClinicalDataControl bcdc = c as BaseClinicalDataControl;
            if (bcdc == null || !bcdc.OverrideReadOnly)
            {
                if (c is TextBox)
                {
                    ((TextBox)c).Enabled = false;
                }
                else if (c is RadioButton)
                {
                    ((RadioButton)c).Enabled = false;
                }
                else if (c is DropDownList)
                {
                    ((DropDownList)c).Enabled = false;
                }
                else if (c is CheckBox)
                {
                    ((CheckBox)c).Enabled = false;
                }
                else if (c is RadioButtonList)
                {
                    ((RadioButtonList)c).Enabled = false;
                }
                else if (c is CheckBoxList)
                {
                    ((CheckBoxList)c).Enabled = false;
                }
                else if (c is RequiredFieldValidator)
                {
                    ((RequiredFieldValidator)c).Enabled = false;
                }
                else
                {
                    //break
                }

                if (c.HasControls())
                {
                    MakeControlsReadOnly(c.Controls);
                }
            }
        }
    }
    private Boolean isQueryStringValid()
    {
        //this iterates over querysting parameters
        //this will handle known querystring parameters and validate range, if any not valid range throw exception
        //if unknown parameter is found, throw exception

        Boolean _isValid = true;

        NameObjectCollectionBase.KeysCollection qryString = Request.QueryString.Keys ;
        
        foreach (string qryStringKey in qryString)
        {
            switch (qryStringKey)
            {
                case "id":
                    _isValid = Helpers.isInteger(Request.QueryString[qryStringKey]);
                    break;
                case "statusId":
                    _isValid = Helpers.isInteger(Request.QueryString[qryStringKey]);
                    break;
                case "searchText":
                    _isValid = Helpers.isString(Request.QueryString[qryStringKey]);
                    break;
                case "refId":
                    _isValid = Helpers.isString(Request.QueryString[qryStringKey]);
                    break;
                case "encId":
                    _isValid = Helpers.isString(Request.QueryString[qryStringKey]);
                    break;
                case "status":
                    _isValid = Helpers.isString(Request.QueryString[qryStringKey]);
                    break;
                default:
                    
                    //do something here   
                    _isValid = false;   
                    break;
                    //throw new Exception("Unknown querystring parameter. " + qryStringKey);
            }

            if (!_isValid) 
            {
                break;
            }
        }

        return _isValid;
    }

    #endregion Private Methods

    #region Protected Methods
    protected void ClearAllControlValues(ControlCollection controls)
    {
        foreach (Control c in controls)
        {
            if (c is TextBox)
            {
                ((TextBox)c).Text = "";
            }
            else if (c is RadioButton)
            {
                ((RadioButton)c).Checked = false;
            }
            else if (c is DropDownList)
            {
                ((DropDownList)c).ClearSelection();
            }
            else if (c is CheckBox)
            {
                ((CheckBox)c).Checked = false;
            }
            else if (c is RadioButtonList)
            {
                ((RadioButtonList)c).ClearSelection();
            }
            else if (c is CheckBoxList)
            {
                ((CheckBoxList)c).ClearSelection();
            }
            else
            {
                //break
            }

            if (c.HasControls())
            {
                ClearAllControlValues(c.Controls);
            }
        }
    }
    protected void manager_BOSaveSuccess(object sender, BaseBO.BOSaveSuccessEventArgs e)
    {
        if (!_errorOnSavedReceived)
        {
            BusinessObjectSaveEventReceived = true;
        }
        //IsSaveSuccess = (e.SaveStatusArg == BaseBO.SaveStatus.SaveSuccess);
    }

    /// <summary>
    /// This method will set standard control properties for all controls on the page.  Sets tabindex and validator control properties for 508
    /// </summary>
    /// <param name="controls"></param>
    protected void SetStandardControlProperties(ControlCollection controls)
    {
        //Int16 tabIndex = 1;

        foreach (Control c in controls)
        {
            //if (c is TextBox
            //    || c is RadioButton
            //    || c is DropDownList
            //    || c is CheckBox
            //    || c is RadioButtonList
            //    || c is CheckBoxList
            //    || c is Button
            //    || c is Label
            //    || c is FileUpload 
            //    || c is ListBox
            //    || c is HtmlInputFile
            //    || c is HtmlInputPassword
            //    || c is HtmlInputText
            //    || c is HtmlSelect
            //    || c is HtmlTextArea

            //    || c is RequiredFieldValidator
            //    || c is RangeValidator
            //    || c is RegularExpressionValidator
            //    || c is CompareValidator
            //    || c is CustomValidator)
            //{
            //    Type type = c.GetType();


            //    PropertyInfo propTabIndex = type.GetProperty("TabIndex");
            //    if (propTabIndex != null) propTabIndex.SetValue(c, tabIndex++, null);
            //}

            if (c is RequiredFieldValidator 
                || c is RangeValidator
                || c is RegularExpressionValidator
                || c is CompareValidator
                || c is CustomValidator
                )
            {
                Type type = c.GetType();

                PropertyInfo propSetFocusOnError = type.GetProperty("SetFocusOnError");
                if (propSetFocusOnError != null) propSetFocusOnError.SetValue(c, true, null);

                PropertyInfo propDisplay = type.GetProperty("Display");
                if (propDisplay != null) propDisplay.SetValue(c, ValidatorDisplay.Dynamic, null);

                PropertyInfo propEnableClientScript = type.GetProperty("EnableClientScript");
                if (propEnableClientScript != null) propEnableClientScript.SetValue(c, true, null);
                PropertyInfo propSetCssClass = type.GetProperty("CssClass");
                if (propSetCssClass != null) propSetCssClass.SetValue(c, "ValidatorClass", null);

            }
           

            if (c.HasControls())
            {
                SetStandardControlProperties(c.Controls);
            }
        }
    }
    protected void CreateLoggingRecord(int objectId, DVEIRUserActivityType userActivityType, string description)
    {
        try
        {
            _currentUser.SessionLog(objectId,
                                    (int)userActivityType,
                                    description,
                                    Request.RawUrl);
        }
        catch
        {
            Response.Redirect("ErrorPage.aspx?id=505");
        }
    }
    protected void CreateLoggingRecord(int objectId, string activityCode, string description)
    {
        DVEIRUserActivityType userActivityType = DVEIRUserActivityType.REFERRAL_OPEN;

        switch (activityCode)
        {
            case "DVEIR MARKED READY FOR REVIEW":
                userActivityType = DVEIRUserActivityType.REFERRAL_SEND_FOR_REVIEW;
                break;
            case "DVEIR SENT BACK TO ABSTRACTOR":
                userActivityType = DVEIRUserActivityType.REFERRAL_SEND_BACK_TO_ABSTRACTOR;
                break;
            case "DVEIR MARKED READY TO SEND TO DVIER":
                userActivityType = DVEIRUserActivityType.REFERRAL_READY_TO_SEND_TO_DVEIR;
                break;
        }

        try
        {
            _currentUser.SessionLog(objectId,
                                    (int)userActivityType,
                                    description,
                                    Request.RawUrl);
        }
        catch
        {
            Response.Redirect("ErrorPage.aspx?id=505");
        }
    }
    #endregion Protected Methods
}