﻿using System;
using System.Collections.Generic;
using System.Linq;
using DataAccess;
using ViewModels.Exam;

namespace Managers.Exam
{
    /// <summary>
    /// Summary description for ExamViewManager
    /// </summary>
    public class AngleRecessionEventViewManager : BaseEventViewManager<AngleRecessionEventModel>
    {
        public override AngleRecessionEventModel GetDefaultViewModel()
        {
            var viewModel = new AngleRecessionEventModel();

            IEnumerable<QUESTION> categoryQuestionList = DataAccessManager.GetQuestionsByCategory(viewModel.CategoryCode).ToList();

            IEnumerable<LATERALITY> lateralityList = DataAccessManager.GetAllLataralities().ToList();

            viewModel.IsLateralityLocked = false;
            viewModel.SelectedLaterality = MapToViewModelLaterality(lateralityList, BaseEventModel.LateralityCodes.NotSpecified);
            viewModel.AvailableLateralities.AddRange(MapToViewModelLateralities(lateralityList));

            viewModel.TestPerformed = MapToSingleSelectQuestion(
                categoryQuestionList,
                AngleRecessionEventModel.QuestionCodes.TestPerformed, 
                AngleRecessionEventModel.ChoiceCodes.TestPerformed.NotDocumented);

            viewModel.TestResult = MapToSingleSelectQuestion(
                categoryQuestionList,
                AngleRecessionEventModel.QuestionCodes.TestResult,
                AngleRecessionEventModel.ChoiceCodes.TestResult.Normal);

            viewModel.EventComment = MapToTextQuestion(categoryQuestionList, AngleRecessionEventModel.QuestionCodes.EventComment);

            viewModel.VisitData = MapToMultiSelectQuestion(categoryQuestionList, AngleRecessionEventModel.QuestionCodes.VisitData);

            viewModel.ClockHours =
                MapToTextQuestionsMultiSelectSub(categoryQuestionList, viewModel.VisitData, AngleRecessionEventModel.QuestionCodes.ClockHourSuffix);

            return viewModel;
        }

        public override AngleRecessionEventModel GetLoadedViewModel(int patientId, int encounterId, string lateralityCode)
        {
            var viewModel = new AngleRecessionEventModel();
            IEnumerable<QUESTION> categoryQuestionList = DataAccessManager.GetQuestionsByCategory(viewModel.CategoryCode).ToList();
            IEnumerable<LATERALITY> lateralityList = DataAccessManager.GetAllLataralities().ToList();
            IEnumerable<RESPONSE> responseList =
                DataAccessManager.GetResponseList(patientId, encounterId, lateralityCode, viewModel.CategoryCode).ToList();

            if (responseList.Any())
            {
                RESPONSE firstResponse = responseList.First();
                DateTime eventDateTime = firstResponse.CREATED ?? DateTime.Now;

                viewModel.EventDate = eventDateTime;
                viewModel.IsLateralityLocked = true;
                viewModel.SelectedLaterality = MapToViewModelLaterality(firstResponse.LATERALITY);
                viewModel.AvailableLateralities.AddRange(MapToViewModelLateralities(lateralityList));

                viewModel.TestPerformed = 
                    MapToSingleSelectQuestion(categoryQuestionList, responseList, AngleRecessionEventModel.QuestionCodes.TestPerformed);
                viewModel.TestResult = 
                    MapToSingleSelectQuestion(categoryQuestionList, responseList, AngleRecessionEventModel.QuestionCodes.TestResult);

                viewModel.EventComment = 
                    MapToTextQuestion(categoryQuestionList, responseList, AngleRecessionEventModel.QuestionCodes.EventComment);

                viewModel.VisitData =
                    MapToMultiSelectQuestion(categoryQuestionList, responseList, AngleRecessionEventModel.QuestionCodes.VisitData);

                viewModel.ClockHours =
                    MapToTextQuestionsMultiSelectSub(categoryQuestionList, responseList, viewModel.VisitData, AngleRecessionEventModel.QuestionCodes.ClockHourSuffix);
            }
            else
            {
                viewModel = GetDefaultViewModel();
            }

            return viewModel;
        }

        public override void Save(AngleRecessionEventModel result, int patientId, int encounterId)
        {
            try
            {
                // Create Responses
                string lateralityCode = result.SelectedLaterality.Code;
                string categoryCode = result.CategoryCode;

                // Find Responses to remove
                List<int> responsesToBeRemoved =
                    DataAccessManager.GetResponseList(patientId, encounterId, lateralityCode, categoryCode)
                    .Select(r => r.RESPONSE_ID)
                    .ToList();

                // Create Responses To Added
                List<RESPONSE> responsesToBeAdded = new List<RESPONSE>();

                // Event Comment
                responsesToBeAdded.Add(MapTextQuestionToResponse(result.EventComment, patientId, encounterId, lateralityCode));

                // Test Performed
                responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(result.TestPerformed, patientId, encounterId, lateralityCode));


                // Only procede if is performed
                if (result.TestPerformed.IsAnswered &&
                    AngleRecessionEventModel.ShowTestDetailValues.Any(v => v == result.TestPerformed.SelectedChoice.Code))
                {
                    // Test Result
                    responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(result.TestResult, patientId, encounterId, lateralityCode));

                    // Visit Data
                    responsesToBeAdded.AddRange(MapMultiSelectQuestionToResponse(result.VisitData, patientId, encounterId, lateralityCode));

                    // Clock Hours
                    responsesToBeAdded.AddRange(MapTextQuestionMultiSelectSubsToResponses(result.ClockHours, result.VisitData, patientId, encounterId, lateralityCode));
                }

                // Save the changes
                DataAccessManager.SaveAndReplaceResponses(responsesToBeRemoved, responsesToBeAdded);
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex);
            }
        }
    }
}