﻿using System;
using System.Collections.Generic;
using System.Linq;
using DataAccess;
using ViewModels.Exam;

namespace Managers.Exam
{
    /// <summary>
    /// Summary description for ColorVisionEventViewManager
    /// </summary>
    public class ColorVisionEventViewManager : BaseEventViewManager<ColorVisionEventModel>
    {
        public override ColorVisionEventModel GetDefaultViewModel()
        {
            var viewModel = new ColorVisionEventModel();
            IEnumerable<QUESTION> categoryQuestionList = DataAccessManager.GetQuestionsByCategory(viewModel.CategoryCode).ToList();
            IEnumerable<LATERALITY> lateralityList = DataAccessManager.GetAllLataralities().ToList();

            // Defaults to both eyes
            viewModel.IsLateralityLocked = true;
            viewModel.AvailableLateralities.Add(MapToViewModelLaterality(lateralityList, BaseEventModel.LateralityCodes.BothEyes));
            viewModel.SelectedLaterality = MapToViewModelLaterality(lateralityList, BaseEventModel.LateralityCodes.BothEyes);

            viewModel.TestPerformed = MapToSingleSelectQuestion(
                categoryQuestionList,
                ColorVisionEventModel.QuestionCodes.TestPerformed,
                ColorVisionEventModel.ChoiceCodes.TestPerformed.PIP);

            viewModel.EventComment = MapToTextQuestion(categoryQuestionList, ColorVisionEventModel.QuestionCodes.EventComment);

            viewModel.NumberCorrect = MapToTextQuestion(categoryQuestionList, ColorVisionEventModel.QuestionCodes.NumberCorrect);

            viewModel.TotalNumber = MapToTextQuestion(categoryQuestionList, ColorVisionEventModel.QuestionCodes.TotalNumber);

            return viewModel;
        }

        public override ColorVisionEventModel GetLoadedViewModel(int patientId, int encounterId, string lateralityCode)
        {
            var viewModel = new ColorVisionEventModel();
            IEnumerable<QUESTION> categoryQuestionList = DataAccessManager.GetQuestionsByCategory(viewModel.CategoryCode).ToList();
            IEnumerable<LATERALITY> lateralityList = DataAccessManager.GetAllLataralities().ToList();
            IEnumerable<RESPONSE> responseList =
                DataAccessManager.GetResponseList(patientId, encounterId, lateralityCode, viewModel.CategoryCode).ToList();

            if (responseList.Any())
            {
                RESPONSE firstResponse = responseList.First();
                DateTime eventDateTime = firstResponse.CREATED ?? DateTime.Now;

                viewModel.EventDate = eventDateTime;
                viewModel.IsLateralityLocked = true;
                viewModel.SelectedLaterality = MapToViewModelLaterality(firstResponse.LATERALITY);
                viewModel.AvailableLateralities.AddRange(MapToViewModelLateralities(lateralityList));
                viewModel.TestPerformed =
                    MapToSingleSelectQuestion(categoryQuestionList, responseList, ColorVisionEventModel.QuestionCodes.TestPerformed);

                viewModel.EventComment =
                    MapToTextQuestion(categoryQuestionList, responseList, ColorVisionEventModel.QuestionCodes.EventComment);

                viewModel.NumberCorrect =
                    MapToTextQuestion(categoryQuestionList, responseList, ColorVisionEventModel.QuestionCodes.NumberCorrect);

                viewModel.TotalNumber =
                    MapToTextQuestion(categoryQuestionList, responseList, ColorVisionEventModel.QuestionCodes.TotalNumber);
            }
            else
            {
                viewModel = GetDefaultViewModel();
            }

            return viewModel;
        }

        public override void Save(ColorVisionEventModel result, int patientId, int encounterId)
        {
            try
            {
                // Create Responses
                string lateralityCode = result.SelectedLaterality.Code;
                string categoryCode = result.CategoryCode;

                // Find Responses to remove
                List<int> responsesToBeRemoved =
                    DataAccessManager.GetResponseList(patientId, encounterId, lateralityCode, categoryCode)
                    .Select(r => r.RESPONSE_ID)
                    .ToList();

                // Create Responses To Added
                List<RESPONSE> responsesToBeAdded = new List<RESPONSE>();

                // Test Performed
                responsesToBeAdded.Add(MapSingleSelectQuestionToResponse(result.TestPerformed, patientId, encounterId, lateralityCode));

                // Event Comment
                responsesToBeAdded.Add(MapTextQuestionToResponse(result.EventComment, patientId, encounterId, lateralityCode));

                if (result.TestPerformed.IsAnswered &&
                    ColorVisionEventModel.ShowTestDetailValues.Any(v => v == result.TestPerformed.SelectedChoice.Code))
                {
                    // Number Correct
                    responsesToBeAdded.Add(MapTextQuestionToResponse(result.NumberCorrect, patientId, encounterId, lateralityCode));

                    // Total Number
                    responsesToBeAdded.Add(MapTextQuestionToResponse(result.NumberCorrect, patientId, encounterId, lateralityCode));
                }

                // Save the changes
                DataAccessManager.SaveAndReplaceResponses(responsesToBeRemoved, responsesToBeAdded);
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex);
            }
        }
    }
}