﻿using System;
using System.Collections.Generic;

namespace ViewModels.Exam
{
    /// <summary>
    /// Summary description for Visual Acuity Event Model
    /// </summary>
    [Serializable]
    public class VisualAcuityEventModel : BaseEventModel
    {
        public override string CategoryCode
        {
            get { return "VisualAcuity"; }
        }

        public bool IsUSScale
        {
            get { return Scale.IsAnswered && Scale.SelectedChoice.Code == ChoiceCodes.Scale.USScale; }
        }

        public bool IsMetricScale
        {
            get { return Scale.IsAnswered && Scale.SelectedChoice.Code == ChoiceCodes.Scale.MetricScale; }
        }

        public bool IsJaegerScale
        {
            get { return Scale.IsAnswered && Scale.SelectedChoice.Code == ChoiceCodes.Scale.JaegerScale; }
        }

        public bool IsDecimalScale
        {
            get { return Scale.IsAnswered && Scale.SelectedChoice.Code == ChoiceCodes.Scale.DecimalScale; }
        }

        public bool IsMUnitScale
        {
            get { return Scale.IsAnswered && Scale.SelectedChoice.Code == ChoiceCodes.Scale.MUnitScale; }
        }

        public bool IsNonNumericScale
        {
            get { return Scale.IsAnswered && Scale.SelectedChoice.Code == ChoiceCodes.Scale.NonNumericScale; }
        }

        public SingleSelectQuestion Scale { get; set; }

        public YesNoQuestion USChartDistance { get; set; }

        public SingleSelectQuestion USAcuity { get; set; }

        public YesNoQuestion USDifferentDistance { get; set; }

        public TextQuestion USPerformDistance { get; set; }

        public SingleSelectQuestion USAdjustment { get; set; }

        public SingleSelectQuestion USUnits { get; set; }

        public YesNoQuestion MetricChartDistance { get; set; }

        public TextQuestion MetricAcuity { get; set; }

        public YesNoQuestion MetricDifferentDistance { get; set; }

        public SingleSelectQuestion MetricPerformDistance { get; set; }

        public SingleSelectQuestion MetricAdjustment { get; set; }

        public SingleSelectQuestion MetricUnits { get; set; }

        public TextQuestion JaegerPerformDistance { get; set; }

        public SingleSelectQuestion JaegerAcuity { get; set; }

        public SingleSelectQuestion JaegerAdjustment { get; set; }

        public SingleSelectQuestion JaegerUnits { get; set; }

        public SingleSelectQuestion JaegerChartType { get; set; }

        public SingleSelectQuestion JaegerPrintSamples { get; set; }

        public TextQuestion DecimalAcuity { get; set; }

        public SingleSelectQuestion DecimalAdjustment { get; set; }

        public SingleSelectQuestion DecimalUnits { get; set; }

        public TextQuestion MUnitAcuity { get; set; }

        public SingleSelectQuestion MUnitAdjustment { get; set; }

        public SingleSelectQuestion MUnitUnits { get; set; }

        public MultiSelectQuestion NonNumericAcuity { get; set; }

        public SingleSelectQuestion Distance { get; set; }

        public SingleSelectQuestion Correction { get; set; }

        public SingleSelectQuestion ExamMethod { get; set; }

        public SingleSelectQuestion ChartType { get; set; }

        public SingleSelectQuestion PrintSample { get; set; }

        public static IEnumerable<string> ShowTestDetailValues
        {
            get
            {
                return new[]
                {
                    ChoiceCodes.TestPerformed.Yes
                };
            }
        }

        public static IEnumerable<string> EnableNonNumericAcuityValidation
        {
            get
            {
                return new[]
                {
                    ChoiceCodes.NonNumericScaleAcuity.CountingFingerIn,
                    ChoiceCodes.NonNumericScaleAcuity.CountingFingerFt,
                    ChoiceCodes.NonNumericScaleAcuity.CountingFingerCm,
                    ChoiceCodes.NonNumericScaleAcuity.CountingFingerM,
                    ChoiceCodes.NonNumericScaleAcuity.HandMotionIn,
                    ChoiceCodes.NonNumericScaleAcuity.HandMotionFt,
                    ChoiceCodes.NonNumericScaleAcuity.HandMotionCm,
                    ChoiceCodes.NonNumericScaleAcuity.HandMotionM
                };
            }
        } 

        public static class QuestionCodes
        {
            public static string TestPerformed = "VA_TP";

            public static string TestResult = "VA_TR";

            public static string EventComment = "VA_C";

            public static string Scale = "VA_S";

            public static string USChartDistance = "VA_USS_NCD";
            public static string USAcuity = "VA_USS_A";
            public static string USDifferentDistance = "VA_USS_TP_DD";
            public static string USPerformDistance = "VA_USS_PD";
            public static string USAdjustment = "VA_USS_AD";
            public static string USUnits = "VA_USS_U";

            public static string MetricChartDistance = "VA_MS_NCD";
            public static string MetricAcuity = "VA_MS_A";
            public static string MetricDifferentDistance = "VA_MS_TP_DD";
            public static string MetricPerformDistance = "VA_MS_PD";
            public static string MetricAdjustment = "VA_MS_AD";
            public static string MetricUnits = "VA_MS_U";

            public static string JaegerPerformDistance = "VA_JS_PD";
            public static string JaegerAcuity = "VA_JS_A";
            public static string JaegerAdjustment = "VA_JS_AD";
            public static string JaegerUnits = "VA_JS_U";
            public static string JaegerChartType = "VA_JS_CT";
            public static string JaegerPrintSamples = "VA_JS_PS";

            public static string DecimalAcuity = "VA_DS_A";
            public static string DecimalAdjustment = "VA_DS_AD";
            public static string DecimalUnits = "VA_DS_U";

            public static string MUnitAcuity = "VA_MUS_A";
            public static string MUnitAdjustment = "VA_MUS_AD";
            public static string MUnitUnits = "VA_MUS_U";

            public static string NonNumericAcuity = "VA_NNS_A";

            public static string Distance = "VA_D";
            public static string Correction = "VA_CO";
            public static string ExamMethod = "VA_EM";
            public static string ChartType = "VA_CT";
            public static string PrintSample = "VA_PS";
        }

        public static class ChoiceCodes
        {
            public static class TestPerformed
            {
                public static string Yes = "VA_TP_Y";

                public static string NotDocumented = "VA_TP_ND";
            }

            public static class TestResult
            {
                public static string Normal = "VA_TR_N";

                public static string Abnormal = "VA_TR_A";
            }

            public static class Scale
            {
                public static string USScale = "VA_S1";

                public static string MetricScale = "VA_S2";

                public static string JaegerScale = "VA_S3";

                public static string DecimalScale = "VA_S4";

                public static string MUnitScale = "VA_S5";

                public static string NonNumericScale = "VA_S6";
            }

            public static class NonNumericScaleAcuity
            {
                public static string CountingFingerIn = "VA_NNS_A3";

                public static string CountingFingerFt = "VA_NNS_A4";

                public static string CountingFingerCm = "VA_NNS_A5";

                public static string CountingFingerM = "VA_NNS_A6";

                public static string HandMotionIn = "VA_NNS_A7";

                public static string HandMotionFt = "VA_NNS_A8";

                public static string HandMotionCm = "VA_NNS_A8";

                public static string HandMotionM = "VA_NNS_A9";
            }

            public static class JaegerChartType
            {
                public static string Jaeger = "VA_JS_CT1";
            }

            public static class JaegerPrintSample
            {
                public static string Jaeger = "VA_JS_PS1";
            }
        }
    }
}