﻿using System;
using System.Collections.Generic;
using System.Linq;

namespace ViewModels
{
    /// <summary>
    /// The Mulit Select Question
    /// </summary>
    [Serializable]
    public class MultiSelectQuestion : BaseQuestion
    {
 
        /// <summary>Gets the selected choices.</summary>
        /// <value>The selected choices.</value>
        public List<Choice> SelectedChoices { get; private set; }

        /// <summary>Gets all choices.</summary>
        /// <value>All choices.</value>
        public List<Choice> AllChoices { get; private set; }

        /// <summary>Gets the available choices ordered.</summary>
        /// <value>The available choices ordered.</value>
        public IEnumerable<Choice> AvailableChoicesOrdered
        {
            get
            {
                // Include only choices that do not exisit
                return AllChoices
                    .Where(c => !SelectedChoices.Exists(selectedChoice => c.Code == selectedChoice.Code))
                    .OrderBy(c => c.SortOrder);
            }
        }

        /// <summary>Gets the selected choices orderd.</summary>
        /// <value>The selected choices orderd.</value>
        public IEnumerable<Choice> SelectedChoicesOrderd
        {
            get { return SelectedChoices.OrderBy(c => c.SortOrder); }
        } 

        /// <summary>Initializes a new instance of the <see cref="MultiSelectQuestion"/> class.</summary>
        public MultiSelectQuestion()
        {
            AllChoices = new List<Choice>();
            SelectedChoices = new List<Choice>();
        }

        public override bool IsAnswered
        {
            get { return SelectedChoices.Any(); }
        }
    }
}