﻿using DataAccess;
using Factories.EncounterOperations;
using Managers.Encounter;
using Managers.Referral;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using Telerik.Web.UI;

public partial class Encounter : BasePage
{
    #region Member Variables

    protected int referralId;
    protected int encounterId;
    protected ER_ENCOUNTER_MOD encounter;

    #endregion

    /// <summary>
    /// Default Page Load
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void Page_Load(object sender, EventArgs e)
    {
        try
        {
            #region Query String Parameter Parsing

            if (!String.IsNullOrEmpty(Request.QueryString.Get("refId")))
            {
                referralId = Convert.ToInt32(Request.QueryString.Get("refId"));
            }
            else
            {
                throw new Exception("Missing Required Parameter - referral id");
            }

            if (!String.IsNullOrEmpty(Request.QueryString.Get("encId")))
            {
                encounterId = Convert.ToInt32(Request.QueryString.Get("encId"));
            }
            else
            {
                throw new Exception("Missing Required Parameter - referral id");
            }

            #endregion

            #region Load Encounter object and call EncounterOperationAvailabilityFactory

            loadPageData();

            #endregion

        }
        catch (Exception ex)
        {
            Logging.WriteLogToFile(ex.Message);
        }
    }

    #region Public Overrides

    public override string TopMenuName()
    {
        return "My Tasks";
    }

    public override string MenuItemName()
    {
        return "DVEIR_REFERRAL";
    }

    public override string AppObjectCode()
    {
        return "DVEIR_REFERRAL_LIST";
    }

    #endregion

    /// <summary>
    /// Fired on Toolbar option clicked.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void rtbAvailalbleOperations_ButtonClick(object sender, Telerik.Web.UI.RadToolBarEventArgs e)
    {
        try
        {
            RadToolBarItem toolBarItem = e.Item;

            switch (Int32.Parse(toolBarItem.Value))
            {
                case EncounterOperationSelectionValues.SAVE:
                    updateEncounter(); // update values in encounter obj
                    EncounterDataManager.SaveEncounter(CurrentUser.UserId, CurrentUser.UserName, encounter);

                    loadPageData();
                    break;
                case EncounterOperationSelectionValues.READY_FOR_REVIEW:
                    updateEncounter();
                    EncounterDataManager.SaveEncounter(CurrentUser.UserId, CurrentUser.UserName, encounter);
                    EncounterDataManager.UpdateEncounterStatus(CurrentUser.UserName, encounterId, EncounterStatusType.REVIEW);

                    Response.Redirect("ReferralEncounters.aspx?id=" + referralId);
                    break;
                case EncounterOperationSelectionValues.RETURN_TO_ABSTRACTOR:

                    updateEncounter();
                    EncounterDataManager.SaveEncounter(CurrentUser.UserId, CurrentUser.UserName, encounter);
                    EncounterDataManager.UpdateEncounterStatus(CurrentUser.UserName, encounterId, EncounterStatusType.SEND_BACK);

                    Response.Redirect("ReferralEncounters.aspx?id=" + referralId);
                    break;
                case EncounterOperationSelectionValues.SEND_TO_DVEIVR:
                    updateEncounter();
                    EncounterDataManager.SaveEncounter(CurrentUser.UserId, CurrentUser.UserName, encounter);
                    EncounterDataManager.UpdateEncounterStatus(CurrentUser.UserName, encounterId, EncounterStatusType.READY_SEND);

                    Response.Redirect("ReferralEncounters.aspx?id=" + referralId);
                    break;
                case EncounterOperationSelectionValues.CANCEL:
                    ((RegistryNoLeftMenu)Page.Master).SetDirtyFlag(false);
                    Response.Redirect("ReferralEncounters.aspx?id=" + referralId);
                    break;
                default:
                    // unacceptable entry
                    break;
            }
        }
        catch (Exception ex)
        {
            Logging.WriteLogToFile("Error Processing Encounter Operation: " + ex.Message);
        }

        //toolBarItem.Value
    }

    protected void CommentListing_OnReviewCommentListSelection(object sender, GridCommandEventArgs e)
    {
        if (e.CommandName == "Details")
        {
            int rowIndex = Convert.ToInt32(e.CommandArgument);
            int commentId = (int)e.Item.OwnerTableView.DataKeyValues[rowIndex]["ID"];
            string quesCode = (string)e.Item.OwnerTableView.DataKeyValues[rowIndex]["QUESTION_CODE"];
            string lateralityCode = (string)e.Item.OwnerTableView.DataKeyValues[rowIndex]["LATERALITY_CODE"];
            int anchorId = (int)e.Item.OwnerTableView.DataKeyValues[rowIndex]["ANCHOR_ID"];

            if (anchorId == ReviewCommentAnchorType.ENCOUNTER_EXAM_TAB)
            {
                rtsTabStrip.SelectedIndex = ReviewCommentAnchorType.ENCOUNTER_EXAM_TAB - 1;
                rmpTabContent.SelectedIndex = ReviewCommentAnchorType.ENCOUNTER_EXAM_TAB - 1;
                ExamTabUserControl.LoadEvent(quesCode, lateralityCode, true);
            }
            else
            {
                List<controls_ReviewComment_ReviewerCommentControl> allControls = new List<controls_ReviewComment_ReviewerCommentControl>();
                EncounterDataManager.GetControlList<controls_ReviewComment_ReviewerCommentControl>(this.Controls, allControls);
                foreach (controls_ReviewComment_ReviewerCommentControl rcc in allControls)
                {
                    if (rcc.QuestionCode == quesCode)
                    {

                        string test = "";

                        // determine page / tab to display
                        switch (anchorId)
                        {
                            case 1:
                                // general tab
                                rtsTabStrip.SelectedIndex = ReviewCommentAnchorType.ENCOUNTER_GENERAL_TAB - 1;
                                rmpTabContent.SelectedIndex = ReviewCommentAnchorType.ENCOUNTER_GENERAL_TAB - 1;

                                // get control
                                rcc.ShowCommentsOnLoad = true;
                                rcc.SetFocusOnLoad = true;
                                rcc.IsSelectedFromReviewGrid = true;
                                rcc.LoadControlsFromObject();
                                return;
                                break;
                            case 2:
                                // provider tab
                                rtsTabStrip.SelectedIndex = ReviewCommentAnchorType.ENCOUNTER_PROVIDER_TAB - 1;
                                rmpTabContent.SelectedIndex = ReviewCommentAnchorType.ENCOUNTER_PROVIDER_TAB - 1;

                                // get control
                                rcc.ShowCommentsOnLoad = true;
                                rcc.SetFocusOnLoad = true;
                                rcc.IsSelectedFromReviewGrid = true;
                                rcc.LoadControlsFromObject();
                                return;
                                break;
                            case 3:
                                // chief complaint tab
                                rtsTabStrip.SelectedIndex = ReviewCommentAnchorType.ENCOUNTER_CHIEF_COMPLAINT_TAB - 1;
                                rmpTabContent.SelectedIndex = ReviewCommentAnchorType.ENCOUNTER_CHIEF_COMPLAINT_TAB - 1;

                                // get control
                                rcc.ShowCommentsOnLoad = true;
                                rcc.SetFocusOnLoad = true;
                                rcc.IsSelectedFromReviewGrid = true;
                                rcc.LoadControlsFromObject();

                                return;
                                break;
                            case 4:
                                // Exam Tab
                                //ts = (RadTabStrip)this.Parent.FindControl("rtsTabStrip");
                                //rmp = (RadMultiPage)this.Parent.FindControl("rmpTabContent");
                                //ts.SelectedIndex = ReviewCommentAnchorType.ENCOUNTER_EXAM_TAB - 1;
                                //rmp.SelectedIndex = ReviewCommentAnchorType.ENCOUNTER_EXAM_TAB - 1;

                                //// get control
                                //rcc.ShowCommentsOnLoad = true;
                                //rcc.SetFocusOnLoad = true;
                                //rcc.IsSelectedFromReviewGrid = true;
                                //rcc.LoadControlsFromObject();

                                return;
                                break;
                            case 5:
                                // Diagnosis Tab
                                rtsTabStrip.SelectedIndex = ReviewCommentAnchorType.ENCOUNTER_DIAGNOSIS_TAB - 1;
                                rmpTabContent.SelectedIndex = ReviewCommentAnchorType.ENCOUNTER_DIAGNOSIS_TAB - 1;

                                // get control
                                rcc.ShowCommentsOnLoad = true;
                                rcc.SetFocusOnLoad = true;
                                rcc.IsSelectedFromReviewGrid = true;
                                rcc.LoadControlsFromObject();

                                return;

                                break;
                            case 6:
                                // Procedure Tab
                                rtsTabStrip.SelectedIndex = ReviewCommentAnchorType.ENCOUNTER_PROCEDURES_TAB - 1;
                                rmpTabContent.SelectedIndex = ReviewCommentAnchorType.ENCOUNTER_PROCEDURES_TAB - 1;

                                // get control
                                rcc.ShowCommentsOnLoad = true;
                                rcc.SetFocusOnLoad = true;
                                rcc.IsSelectedFromReviewGrid = true;
                                rcc.LoadControlsFromObject();
                                return;
                                break;
                            default:
                                break;
                        }
                    }
                }
            }
        }
    }

    /// <summary>
    /// calls to update the encounter object by reference.
    /// </summary>
    private void updateEncounter()
    {
        // get general data
        EncounterInfo.SaveEncounterInformation(ref encounter);

        // get complaint text
        complaintTabUserControl.SaveChiefComplaint(ref encounter);

        // set diagnosis info
        diagnosisTabControl.SaveEncounterDiagnosisInformation(ref encounter);

        // Save Exam Tab
        ExamTabUserControl.Save();
    }

    /// <summary>
    /// Loads all page data.
    /// </summary>
    private void loadPageData()
    {
        try
        {
            encounter = EncounterDataManager.GetEncounterById(encounterId);
            if (encounter != null)
            {
                bool isEditable = EncounterOperationAvailabilityFactory.ConfigureEncounterOperationMenu(CurrentUser, encounter, ref rtbAvailalbleOperations);

                #region General Encounter Information

                EncounterInfo.IsEditable = isEditable;
                EncounterInfo.ReferralDate = ReferralDataManager.GetReferralDateByEncounterID(encounterId);

                // get review comments
                List<ER_ENCOUNTER_REVIEWCOMMENT> reviewComments = EncounterDataManager.GetAllReviewCommentsByEncounterId(encounterId);
                if (reviewComments != null && reviewComments.Count > 0)
                {
                    // show control
                    CommentListing.ReviewComments = reviewComments;
                    CommentListing.Visible = true;
                }
                else
                {
                    CommentListing.Visible = false;
                }

                #endregion

                #region Provider Tab Information

                ProviderTabControl.IsEditable = isEditable;
                ProviderTabControl.EncounterMod = encounter;

                #endregion

                #region Complaint Tab Information

                //Set Complaint Tab Encounter Information
                complaintTabUserControl.Encounter = encounter;
                complaintTabUserControl.CurrentUser = CurrentUser;

                #endregion

                #region Exam Tab Information

                // Set Exam Tab Parameters
                ExamTabUserControl.Encounter = encounter;
                ExamTabUserControl.CurrentUser = CurrentUser;
                ExamTabUserControl.IsEditable = isEditable;

                #endregion

                #region Diagnosis Tab Information

                // Set Diagnosis Tab Information
                diagnosisTabControl.VisitIdentifier = encounter.UniqueVHAEncounterIDPrimaryKey;
                diagnosisTabControl.IsEditable = isEditable;
                diagnosisTabControl.CurrentUser = CurrentUser;
                diagnosisTabControl.EncounterID = encounter.ER_ENCOUNTER_ID;
                diagnosisTabControl.EncounterStatusID = (encounter.EncounterStatusID != null) ? (int)encounter.EncounterStatusID : 0;

                if (!IsPostBack)
                {
                    diagnosisTabControl.BlindRehabIndicator = encounter.BLIND_REHAB_INDICATOR;
                }

                #endregion

                #region Procedure Tab Information

                // Set Procedure Tab Information
                procedureTabControl.VisitIdentifier = encounter.UniqueVHAEncounterIDPrimaryKey;
                procedureTabControl.IsEditable = isEditable;

                #endregion

                #region Set Breadcrumb Text

                StringBuilder breadCrumb = new StringBuilder();
                breadCrumb.Append("Referrals > ");
                breadCrumb.Append("<a href=\"ReferralEncounters.aspx?id=" + referralId + "\" title=\"Click to go back to encounter listing for referral id: " + referralId + "\" style=\"color:#FFFFFF;\">");
                breadCrumb.Append("Encounters for Referrral (" + referralId + ")</a> > ");
                breadCrumb.Append("Encounter (" + encounterId + ") - ");

                switch (encounter.EncounterStatusID)
                {
                    case EncounterStatusType.NEW:
                        breadCrumb.Append("New");
                        break;
                    case EncounterStatusType.IN_PROC:
                        breadCrumb.Append("In Process");
                        break;
                    case EncounterStatusType.REVIEW:
                        breadCrumb.Append("Ready for Review");
                        break;
                    case EncounterStatusType.SEND_BACK:
                        breadCrumb.Append("Send Back to Abstractor");
                        break;
                    case EncounterStatusType.READY_SEND:
                        breadCrumb.Append("Ready to Send to DVEIVR");
                        break;
                    case EncounterStatusType.COMPLETED:
                        breadCrumb.Append("Completed");
                        break;
                    default:
                        break;
                }

                ((RegistryNoLeftMenu)Page.Master).SetBreadCrumbLabel(breadCrumb.ToString());

                #endregion
            }
            else
            {
                // error - no encounter
                throw new Exception("No Encounter Found Exception");
            }
        }
        catch (Exception ex)
        {
            // TODO - Display error to user
            Logging.WriteLogToFile("Error loading Encounter (" + encounterId + ") for Referral (" + referralId + "): " + ex.Message);
        }
    }

    protected void ExamTabUserControl_OnEventSavedSuccessfully()
    {
        if (encounter.EncounterStatusID == EncounterStatusType.NEW)
        {
            EncounterDataManager.UpdateEncounterStatus(CurrentUser.UserName, encounterId, EncounterStatusType.IN_PROC);
            EncounterOperationAvailabilityFactory.ConfigureEncounterOperationMenu(CurrentUser, encounter, ref rtbAvailalbleOperations);

            // Force Update on the pnl
            RadAjaxManager1.ResponseScripts.Add("__doPostBack('ForcePostback','')");
        }
    }
}