﻿using Managers.Encounter;
using Managers.Referral;
using System;
using System.Data;
using Telerik.Web.UI;

public partial class ReferralEncounters : BasePage
{
    #region Member Variables

    protected int referralId = 0;
    protected string status;

    #endregion

    /// <summary>
    /// Default Page Load method
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void Page_Load(object sender, EventArgs e)
    {
        try
        {
            //status = String.IsNullOrEmpty(Request.QueryString.Get("status")) ? "" : Request.QueryString.Get("status");
            ((RegistryNoLeftMenu)Page.Master).SetDirtyFlag(false);

            if (!String.IsNullOrEmpty(Request.QueryString.Get("id")))
            {
                if (Int32.TryParse(Request.QueryString.Get("id"), out referralId))
                {
                    // set label
                    ((RegistryNoLeftMenu)Page.Master).SetBreadCrumbLabel("<a href=\"\" title=\"Back to Referral List Page\" style=\"color:#FFFFFF;\">Referrals</a> > Encounters for Referrral (" + referralId + ")");

                    if (!IsPostBack)
                    {
                        rgReferralEncounters.DataSource = "";
                        rgReferralEncounters.DataBind();

                        fillReportGrid();
                        rgReferralEncounters.DataBind();

                        // build stations ...
                        //rcbInstitution.DataSource = ReferralDataManager.GetFacilitiesByVisnId(0);
                        //rcbInstitution.DataBind();
                    }
                }

            }
            else
            {
                throw new Exception("Required Parameter Exception");
            }
        }
        catch (Exception ex)
        {
            throw new Exception("Required Parameter Exception", ex.InnerException);
        }
    }

    #region GUI Overrides

    public override string TopMenuName()
    {
        return "Referrals";
    }

    public override string MenuItemName()
    {
        return "DVEIR_REFERRAL";
    }

    public override string AppObjectCode()
    {
        return "DVEIR_REFERRAL_LIST";
    }

    #endregion

    #region GUI Events

    /// <summary>
    /// Encounter Status Change
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void rcbEncounterStatus_SelectedIndexChanged(object sender, RadComboBoxSelectedIndexChangedEventArgs e)
    {
        fillReportGrid();
        rgReferralEncounters.DataBind();
    }

    /// <summary>
    /// Event for Add Encounter Button Click
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void btnAddEncounter_Click(object sender, EventArgs e)
    {
        pnlReferralEncounters.Visible = false;
        pnlAddEncounter.Visible = true;

        lblReferralId.Text = referralId.ToString();
    }

    /// <summary>
    /// Save Encounter Click
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void rbSaveEncounterRecord_Click(object sender, EventArgs e)
    {
        try
        {
            if (Page.IsValid)
            {
                DataTable dt = ReferralDataManager.GetPatientSIDsByReferralId(referralId);
                int patientSid = 0;

                if (dt != null)
                {
                    if (dt.Rows.Count > 0)
                    {
                        // grab first patient sid
                        patientSid = (dt.Rows[0]["PatientSID"] != null) ? Convert.ToInt32(dt.Rows[0]["PatientSID"].ToString()) : 0;
                    }
                }

                // TODO: Validation
                EncounterDataManager.InsertEncounter(CurrentUser.UserId, CurrentUser.UserName, referralId, patientSid, Convert.ToInt32(rcbInstitution.SelectedValue), (DateTime)rdtpVisitDate.SelectedDate, rcbEncounterType.SelectedValue);

                // hide this view - show the other grid and refresh data.

                rgReferralEncounters.DataSource = "";
                rgReferralEncounters.DataBind();

                fillReportGrid();
                rgReferralEncounters.DataBind();

                clearForm();

                pnlReferralEncounters.Visible = true;
                pnlAddEncounter.Visible = false;
            }
            else
            {
                // tell user.
            }

        }
        catch (Exception ex)
        {
            // TODO: Display Error Label for user ... 
            Logging.WriteLogToFile("Error Manually Creating Encounter Record: " + ex.Message);
        }
    }

    /// <summary>
    /// Cancel Encounter creation
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void rbCancelEncounterRecord_Click(object sender, EventArgs e)
    {
        // clear form
        clearForm();

        pnlReferralEncounters.Visible = true;
        pnlAddEncounter.Visible = false;
    }

    #endregion

    #region Grid Events

    /// <summary>
    /// On select button pushed.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void rgReferralEncounters_ItemCommand(object sender, Telerik.Web.UI.GridCommandEventArgs e)
    {
        if (e.CommandName == "Details")
        {
            int rowIndex = Convert.ToInt32(e.CommandArgument);
            int encId = (int)e.Item.OwnerTableView.DataKeyValues[rowIndex]["ER_ENCOUNTER_ID"];
            Response.Redirect(string.Format("Encounter.aspx?refId={0}&encId={1}", referralId, encId));
        }
    }

    /// <summary>
    /// Datasource needed.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void rgReferralEncounters_NeedDataSource(object sender, Telerik.Web.UI.GridNeedDataSourceEventArgs e)
    {
        fillReportGrid();
    }

    #endregion

    #region Private Methods

    /// <summary>
    /// Populates the Grid with Data.
    /// </summary>
    private void fillReportGrid()
    {
        try
        {
            //status
            int statusId = 0;

            status = rcbEncounterStatus.SelectedValue;

            switch (status.ToLower())
            {
                case "all":
                    statusId = 0;
                    break;
                case "new":
                    statusId = 1;
                    break;
                case "inproc":
                    statusId = 2;
                    break;
                case "review":
                    statusId = 3;
                    break;
                case "returned":
                    statusId = 4;
                    break;
                case "send":
                    statusId = 5;
                    break;
                case "completed":
                    statusId = 6;
                    break;
                default:
                    statusId = 0;
                    break;
            }

            rgReferralEncounters.DataSource = ReferralDataManager.GetEncountersByReferralId(referralId, statusId);
        }
        catch (Exception ex)
        {
            rgReferralEncounters.DataSource = null;
        }
    }

    /// <summary>
    /// Clear Add Encounter Form values.
    /// </summary>
    private void clearForm()
    {
        lblReferralId.Text = "";
        rcbInstitution.ClearSelection();
        rcbInstitution.Text = "";
        rdtpVisitDate.Clear();
        rcbEncounterType.ClearSelection();
    }

    #endregion
}