﻿using DataAccess;
using Managers.Referral;
using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Web;
using Telerik.Web.UI;
using VeteransAffairs.Registries.BusinessManager;

public partial class Referrals : BasePage
{
    #region Member Variables

    private UserAccountManager _user = (UserAccountManager)HttpContext.Current.User;
    protected string status;

    #endregion

    /// <summary>
    /// Default Page Load Method
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void Page_Load(object sender, EventArgs e)
    {
        ((RegistryNoLeftMenu)Page.Master).SetDirtyFlag(false);
        // set label
        ((RegistryNoLeftMenu)Page.Master).SetBreadCrumbLabel("Referrals");

        if (!IsPostBack)
        {
            loadVISNList();

            rgReferrals.DataSource = "";
            rgReferrals.DataBind();
        }
    }

    #region GUI Overrides

    public override string TopMenuName()
    {
        return "Referrals";
    }

    public override string MenuItemName()
    {
        return "DVEIR_REFERRAL";
    }

    public override string AppObjectCode()
    {
        return "DVEIR_REFERRAL_LIST";
    }

    #endregion

    #region GUI Events

    /// <summary>
    /// VISN Dropdown selected index changed.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void rcbVISN_SelectedIndexChanged(object sender, Telerik.Web.UI.RadComboBoxSelectedIndexChangedEventArgs e)
    {
        //rgReferrals.DataSource = "";
        //rgReferrals.DataBind();

        if (rcbVISN.SelectedIndex > 0)
        {
            rcbFac.Enabled = true;
            loadFacilityList(rcbVISN.SelectedValue);
        }
        else
        {
            rcbFac.ClearSelection();
            rcbFac.Enabled = false;
        }
    }

    /// <summary>
    /// Facility Dropdown changed ... just ensure datagrid is not showing data until Search is clicked.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void rcbFac_SelectedIndexChanged(object sender, RadComboBoxSelectedIndexChangedEventArgs e)
    {
        //rgReferrals.DataSource = "";
        //rgReferrals.DataBind();
    }

    /// <summary>
    /// Referral Type Changed
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void rcbReferralType_SelectedIndexChanged(object sender, RadComboBoxSelectedIndexChangedEventArgs e)
    {
        //if (rcbVISN.SelectedIndex > 0 || !String.IsNullOrEmpty(rtbPatientNameSSN.Text))
        //{
        //fillReportGrid();
        //rgReferrals.DataBind();
        //}
    }

    /// <summary>
    /// Search Button Clicked
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void rbSearch_Click(object sender, EventArgs e)
    {
        rgReferrals.CurrentPageIndex = 0;
        fillReportGrid();
        rgReferrals.DataBind();
    }

    #endregion

    #region Grid Events

    protected void rgReferrals_NeedDataSource(object sender, Telerik.Web.UI.GridNeedDataSourceEventArgs e)
    {
        //rgReferrals.DataSource = "";
        fillReportGrid();
    }

    protected void rgReferrals_ItemCommand(object sender, Telerik.Web.UI.GridCommandEventArgs e)
    {
        if (e.CommandName == "Details")
        {
            int rowIndex = Convert.ToInt32(e.CommandArgument);
            int refId = (int)e.Item.OwnerTableView.DataKeyValues[rowIndex]["REFERRAL_ID"];
            if (!String.IsNullOrEmpty(status))
            {
                Response.Redirect(string.Format("ReferralEncounters.aspx?id={0}&status={1}", refId, status));
            }
            else
            {
                Response.Redirect(string.Format("ReferralEncounters.aspx?id={0}", refId));
            }
        }
    }

    #endregion

    #region Private Methods

    /// <summary>
    /// Loads the VISN Dropdown List
    /// </summary>
    private void loadVISNList()
    {
        using (var ctx = new MEVIREntities())
        {
            var temp = ctx.usp_GetVISNs();

            List<usp_GetVISNs_Result> visnList = temp.ToList();

            visnList.Insert(0, new usp_GetVISNs_Result() { ID = 0, NAME = "Select a VISN" });

            rcbVISN.DataSource = visnList;
            rcbVISN.DataTextField = "NAME";
            rcbVISN.DataValueField = "ID";
            rcbVISN.DataBind();
        }
    }

    /// <summary>
    /// call sproc to return facility list
    /// </summary>
    /// <param name="visnId"></param>
    private void loadFacilityList(string visnId)
    {
        using (var ctx = new MEVIREntities())
        {
            int visnIdNum = Convert.ToInt32(visnId);

            List<usp_Get_Facilities_by_Visn_ID_Result> facList = ctx.usp_Get_Facilities_by_Visn_ID(visnIdNum).ToList();
            
            facList.Insert(0, new usp_Get_Facilities_by_Visn_ID_Result() { ID = 0, VISTANAME = "Select a Facility" });

            rcbFac.DataSource = facList;
            rcbFac.DataTextField = "VISTANAME";
            rcbFac.DataValueField = "ID";
            rcbFac.DataBind();
        }
    }

    /// <summary>
    /// Populates the Grid with Data.
    /// </summary>
    private void fillReportGrid()
    {
        status = (!String.IsNullOrEmpty(rcbReferralType.SelectedValue)) ? rcbReferralType.SelectedValue : "";

        int statusId = 0;

        switch (status.ToLower())
        {
            case "all":
                statusId = 0;
                break;
            case "new":
                statusId = 1;
                break;
            case "inproc":
                statusId = 2;
                break;
            case "returned":
                statusId = 3;
                break;
            case "review":
                statusId = 4;
                break;
            case "send":
                statusId = 5;
                break;
            case "completed":
                statusId = 6;
                break;
            default:
                statusId = 1;
                break;
        }

        try
        {
            int? visn = (rcbVISN.SelectedIndex > 0) ? (int?)Convert.ToInt32(rcbVISN.SelectedValue) : null;
            int? fac = (rcbFac.SelectedIndex > 0) ? (int?)Convert.ToInt32(rcbFac.SelectedValue) : null;
            string patNameSSN = !String.IsNullOrEmpty(rtbPatientNameSSN.Text) ? rtbPatientNameSSN.Text : null;

            rgReferrals.DataSource = ReferralDataManager.GetReferralsWithCriteria(statusId, visn, fac, patNameSSN);
        }
        catch (Exception ex)
        {
            rgReferrals.DataSource = null;
        }
    }

    #endregion
}