﻿using DataAccess;
using Managers.Encounter;
using System;
using Telerik.Web.UI;
using VeteransAffairs.Registries.BusinessManager;
using ViewModels.EncounterDetail;

public partial class controls_EncounterDiagnosis_EncounterDiagnosisControl : System.Web.UI.UserControl
{
    #region Member Variables

    public bool IsEditable { get; set; }
    public string VisitIdentifier { get; set; }
    public string BlindRehabIndicator 
    { 
        get 
        {
            if (rbBlindRehabYes.Checked)
            {
                return "Y";
            }
            else if (rbBlindRehabNo.Checked)
            {
                return "N";
            }
            else
            {
                return null;
            }
        } 
        set 
        {
            if (value == "Y")
            {
                rbBlindRehabYes.Checked = true;
            }
            else if (value == "N")
            {
                rbBlindRehabNo.Checked = true;
            }
            else
            {
                rbBlindRehabNA.Checked = true;
            }
        } 
    }
    public int EncounterID { get; set; }
    public int EncounterStatusID { get; set; }
    public UserAccountManager CurrentUser { get; set; }

    #endregion

    /// <summary>
    /// Default Page Load
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void Page_Load(object sender, EventArgs e)
    {
        try
        {
            if (!IsPostBack)
            {
                fillReportGrid();
            }

            //rccDiagnosisBlindRehab
            // Review Comments
            if (EncounterID != null && EncounterID > 0)
            {
                rccDiagnosisBlindRehab.EncounterId = EncounterID;
                rccDiagnosisBlindRehab.QuestionCode = "ENC_DIAGNOSIS";
                rccDiagnosisBlindRehab.AnchorId = ReviewCommentAnchorType.ENCOUNTER_DIAGNOSIS_TAB;
                rccDiagnosisBlindRehab.FriendlyLocationName = "Diagnosis > Vision Rehabilitation";
                rccDiagnosisBlindRehab.IsCommentEditable = true;
                //rccChiefComplaint.SetFocusOnLoad
                rccDiagnosisBlindRehab.CurrentUser = CurrentUser;

                rccDiagnosisBlindRehab.EncounterStatusID = (EncounterStatusID != null) ? EncounterStatusID : 0;

                if (!IsPostBack)
                {
                    rccDiagnosisBlindRehab.ShowCommentControl = true;
                    rccDiagnosisBlindRehab.ShowCommentsOnLoad = false;
                    rccDiagnosisBlindRehab.LoadControlsFromObject();
                }
            }

            rbAddDiagnosisEntry.Text = "Add Diagnosis";
            rbAddDiagnosisEntry.ToolTip = "Click to add diagnosis entry to the encounter";
        }
        catch (Exception ex)
        {
            Logging.WriteLogToFile(ex.Message);
        }
    }

    #region Grid Events

    /// <summary>
    /// 
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void rgDiagnosis_ItemCommand(object sender, Telerik.Web.UI.GridCommandEventArgs e)
    {
        GridDataItem item;
        int diagnosisID;

        if (e.CommandName == "EditDiagnosis")
        {
            if (e.Item is GridDataItem)
            {
                item = ((GridDataItem)e.Item);

                diagnosisID = (int)item.GetDataKeyValue("ID");
                hfEncounterDetailID.Value = diagnosisID.ToString();

                try
                {
                    ENCOUNTER_DETAILS detail = EncounterDataManager.GetEncounterDetailById(diagnosisID);
                    if (detail != null)
                    {
                        // populate Add Panel with Edit ... 
                        rgDiagnosis.Visible = false;
                        pnlAddDiagnosisBtn.Visible = false;
                        pnlAddDiagnosisRegion.Visible = true;

                        rtbDetailCode.Text = detail.Value;
                        rtbDescription.Text = detail.Description;

                        rbAddDiagnosisEntry.Text = "Update";
                        rbAddDiagnosisEntry.ToolTip = "Click to update the diagnosis entry";
                    }
                }
                catch (Exception ex)
                {
                    Logging.WriteLogToFile(ex.Message);
                }
            }
        }
        else if (e.CommandName == "DeleteDiagnosis")
        {
            if (e.Item is GridDataItem)
            {
                item = ((GridDataItem)e.Item);

                diagnosisID = (int)item.GetDataKeyValue("ID");

                try
                {
                    EncounterDataManager.DeleteEncounterDetailById(diagnosisID);
                    fillReportGrid();
                }
                catch (Exception ex)
                {
                    Logging.WriteLogToFile(ex.Message);
                }
            }
        }
    }

    /// <summary>
    /// On PreRender show/hide Delete / Edit buttons based on IsEditable
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void rgDiagnosis_PreRender(object sender, EventArgs e)
    {
        rgDiagnosis.MasterTableView.GetColumn("EditDiagnosisColumn").Display = IsEditable;
        rgDiagnosis.MasterTableView.GetColumn("DeleteDiagnosisColumn").Display = IsEditable;
    }

    /// <summary>
    /// 
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void rgDiagnosis_NeedDataSource(object sender, Telerik.Web.UI.GridNeedDataSourceEventArgs e)
    {
        rgDiagnosis.DataSource = "";
    }

    #endregion

    #region GUI Events

    /// <summary>
    /// Add Button Clicked.
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void rbAddDiagnosis_Click(object sender, EventArgs e)
    {
        rgDiagnosis.Visible = false;
        pnlAddDiagnosisBtn.Visible = false;
        pnlAddDiagnosisRegion.Visible = true;
    }

    /// <summary>
    /// Add Entry
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void rbAddDiagnosisEntry_Click(object sender, EventArgs e)
    {

        if (String.IsNullOrEmpty(hfEncounterDetailID.Value))
        {
            // Add entry to the LocalProviderList object ...
            EncounterDataManager.InsertEncounterDetail(VisitIdentifier, EncounterDetailType.ICD, rtbDetailCode.Text, null, rtbDescription.Text);
        }
        else
        {
            // Update
            EncounterDataManager.UpdateEncounterDetail(new ENCOUNTER_DETAILS() { 
                                                            ID = Convert.ToInt32(hfEncounterDetailID.Value), 
                                                            Name = null, 
                                                            Type = EncounterDetailType.ICD, 
                                                            Value = rtbDetailCode.Text, 
                                                            Description = rtbDescription.Text, 
                                                            VisitIdentifier = VisitIdentifier 
                                                            });

            hfEncounterDetailID.Value = null;

            rbAddDiagnosisEntry.Text = "Add Diagnosis";
            rbAddDiagnosisEntry.ToolTip = "Click to add diagnosis entry to the encounter";

        }
        rgDiagnosis.Visible = true;
        pnlAddDiagnosisBtn.Visible = true;

        pnlAddDiagnosisRegion.Visible = false;

        // clear values
        rtbDetailCode.Text = "";
        rtbDescription.Text = "";

        fillReportGrid();
    }

    /// <summary>
    /// Cancel adding entry
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void rbCancelAddDiagnosisEntry_Click(object sender, EventArgs e)
    {
        rgDiagnosis.Visible = true;
        pnlAddDiagnosisBtn.Visible = true;

        // clear values
        rtbDetailCode.Text = "";
        rtbDescription.Text = "";
        hfEncounterDetailID.Value = null;

        rbAddDiagnosisEntry.Text = "Add Diagnosis";
        rbAddDiagnosisEntry.ToolTip = "Click to add diagnosis entry to the encounter";

        pnlAddDiagnosisRegion.Visible = false;
    }

    #endregion

    #region Public Methods

    /// <summary>
    /// Method is called from Save Encounter method
    /// </summary>
    /// <param name="passedEncounter"></param>
    public void SaveEncounterDiagnosisInformation(ref ER_ENCOUNTER_MOD passedEncounter)
    {
        // will really only save the Blind Rehab Entry
        passedEncounter.BLIND_REHAB_INDICATOR = BlindRehabIndicator;
    }

    #endregion

    #region Private Methods

    /// <summary>
    /// Fills the report grid with entries from the db.
    /// </summary>
    private void fillReportGrid()
    {
        pnlAddDiagnosisBtn.Visible = IsEditable;


        rgDiagnosis.DataSource = EncounterDataManager.GetEncounterDetailsByEncounterAndType(VisitIdentifier, EncounterDetailType.ICD);
        rgDiagnosis.DataBind();
    }

    #endregion
}