﻿using DataAccess;
using Managers.Encounter;
using Managers.Referral;
using System;
using System.Data;

public partial class controls_EncounterInformationUserControl : System.Web.UI.UserControl
{
    #region Member Variables 

    public bool IsEditable { get; set; }

    protected int referralId = 0;
    protected int encounterId = 0;
    protected ER_ENCOUNTER_MOD encounter;
    public DateTime? ReferralDate { get; set; }

    #endregion

    /// <summary>
    /// Default Page Load
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void Page_Load(object sender, EventArgs e)
    {
        #region Query String Parameter Parsing

        if (!String.IsNullOrEmpty(Request.QueryString.Get("refId")))
        {
            referralId = Convert.ToInt32(Request.QueryString.Get("refId"));
        }
        else
        {
            throw new Exception("Missing Required Parameter - referral id");
        }

        if (!String.IsNullOrEmpty(Request.QueryString.Get("encId")))
        {
            encounterId = Convert.ToInt32(Request.QueryString.Get("encId"));
        }
        else
        {
            throw new Exception("Missing Required Parameter - referral id");
        }

        #endregion

        if (!IsPostBack)
        {
            // set data initially
            //Set field information
            if (referralId > 0 && encounterId > 0)
            {
                encounter = EncounterDataManager.GetEncounterById(encounterId);

                if (encounter != null)
                {
                    populateDataFields();
                }
            }
        }

        setVisibility();
    }

    #region Public Methods

    /// <summary>
    /// SaveEncounterInformation: receives an ER_ENCOUNTER_MOD object by reference, sets its values and returns.
    /// </summary>
    /// <param name="passedEncounter"></param>
    public void SaveEncounterInformation(ref ER_ENCOUNTER_MOD passedEncounter)
    {
        if (IsEditable) {
            if (Page.IsValid)
            {
                if (passedEncounter != null)
                {
                    passedEncounter.VISIT_DATE = (DateTime)rdtpDateOfEncounterEditable.SelectedDate;
                    passedEncounter.InstitutionID = (rcbInstitution.SelectedValue != "") ? (int?)Convert.ToInt32(rcbInstitution.SelectedValue) : null;

                    DataRow dr = EncounterDataManager.GetFacilityNameAndSta3nById(passedEncounter.InstitutionID);
                    passedEncounter.STA6A = (dr["STATIONNUMBER"] != null) ? dr["STATIONNUMBER"].ToString() : "";

                    passedEncounter.EncounterTypeInOut = rcbEncounterType.SelectedValue;
                }
            }
        }
    }

    #endregion

    #region Private Methods

    /// <summary>
    /// Populate controls
    /// </summary>
    private void populateDataFields()
    {
        lblActualDate.Text = encounter.VISIT_DATE.ToString();
        DataRow dr = EncounterDataManager.GetFacilityNameAndSta3nById(encounter.InstitutionID);

        if (dr != null)
        {
            lblActualFacility.Text = (dr["VISTANAME"] != null) ? dr["VISTANAME"].ToString() : "";
        }

        rdtpDateOfEncounterEditable.SelectedDate = encounter.VISIT_DATE;
        rcbInstitution.SelectedValue = (encounter.InstitutionID != null) ? encounter.InstitutionID.ToString() : null;

        switch (encounter.EncounterTypeInOut)
        {
            case "I":
                lblEncounterType.Text = "Inpatient";
                break;
            case "O":
                lblEncounterType.Text = "Outpatient";
                break;
            default:
                lblEncounterType.Text = "";
                break;
        }

        rcbEncounterType.SelectedValue = encounter.EncounterTypeInOut;

        lblReferralDate.Text = (ReferralDate != null) ? ((DateTime)ReferralDate).ToString() : "";
        lblReferralDateEditable.Text = (ReferralDate != null) ? ((DateTime)ReferralDate).ToString() : "";
    }

    /// <summary>
    /// Determine which controls to show based on IsEditable
    /// </summary>
    private void setVisibility()
    {
        pnlEditableEncounterInfo.Visible = IsEditable;
        pnlReadOnlyEncounterInfo.Visible = !IsEditable;
    }

    #endregion
}