﻿using DataAccess;
using Managers.Encounter;
using System;
using Telerik.Web.UI;
using ViewModels.EncounterDetail;

public partial class controls_EncounterProcedureTab_EncounterProcedureControl : System.Web.UI.UserControl
{
    #region Member Variables

    public bool IsEditable { get; set; }
    public string VisitIdentifier { get; set; }

    #endregion

    /// <summary>
    /// Default Page Load
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void Page_Load(object sender, EventArgs e)
    {
        try
        {
            if (!IsPostBack)
            {
                fillReportGrid();
            }
        }
        catch (Exception ex)
        {
            Logging.WriteLogToFile(ex.Message);
        }
    }

    #region Grid Events

    protected void rgProcedures_NeedDataSource(object sender, Telerik.Web.UI.GridNeedDataSourceEventArgs e)
    {
        rgProcedures.DataSource = "";
    }

    /// <summary>
    /// Grid PreRender
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void rgProcedures_PreRender(object sender, EventArgs e)
    {
        rgProcedures.MasterTableView.GetColumn("DeleteProcedureColumn").Display = IsEditable;
    }

    /// <summary>
    /// Fired on Item Command
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void rgProcedures_ItemCommand(object sender, Telerik.Web.UI.GridCommandEventArgs e)
    {
        GridDataItem item;
        int procedureID;

        if (e.CommandName == "EditProcedure")
        {
            if (e.Item is GridDataItem)
            {
                item = ((GridDataItem)e.Item);

                procedureID = (int)item.GetDataKeyValue("ID");
                hfEncounterDetailID.Value = procedureID.ToString();

                try
                {
                    ENCOUNTER_DETAILS detail = EncounterDataManager.GetEncounterDetailById(procedureID);
                    if (detail != null)
                    {
                        // populate Add Panel with Edit ... 
                        rgProcedures.Visible = false;
                        pnlAddProcedureBtn.Visible = false;
                        pnlAddProcedureRegion.Visible = true;

                        rtbDetailName.Text = detail.Name;
                        rtbDetailCode.Text = detail.Value;
                        rtbDescription.Text = detail.Description;

                        rbAddProcedureEntry.Text = "Update";
                        rbAddProcedureEntry.ToolTip = "Click to update the procedure entry";
                    }
                }
                catch (Exception ex)
                {
                    Logging.WriteLogToFile(ex.Message);
                }
            }
        }
        else if (e.CommandName == "DeleteProcedure")
        {
            if (e.Item is GridDataItem)
            {
                item = ((GridDataItem)e.Item);

                procedureID = (int)item.GetDataKeyValue("ID");

                try
                {
                    EncounterDataManager.DeleteEncounterDetailById(procedureID);
                    fillReportGrid();
                }
                catch (Exception ex)
                {
                    Logging.WriteLogToFile(ex.Message);
                }
            }
        }
    }

    #endregion

    #region GUI Events

    /// <summary>
    /// Add procedure area to show
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void rbAddProcedure_Click(object sender, EventArgs e)
    {
        rgProcedures.Visible = false;
        pnlAddProcedureBtn.Visible = false;
        pnlAddProcedureRegion.Visible = true;
    }

    /// <summary>
    /// Add Procedure to the encounter
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void rbAddProcedureEntry_Click(object sender, EventArgs e)
    {
        if (String.IsNullOrEmpty(hfEncounterDetailID.Value))
        {
            // Add entry to the LocalProviderList object ...
            EncounterDataManager.InsertEncounterDetail(VisitIdentifier, EncounterDetailType.CPT, rtbDetailCode.Text, rtbDetailName.Text, rtbDescription.Text);
        }
        else
        {
            // Update
            EncounterDataManager.UpdateEncounterDetail(new ENCOUNTER_DETAILS()
            {
                ID = Convert.ToInt32(hfEncounterDetailID.Value),
                Name = rtbDetailName.Text,
                Type = EncounterDetailType.CPT,
                Value = rtbDetailCode.Text,
                Description = rtbDescription.Text,
                VisitIdentifier = VisitIdentifier
            });

            hfEncounterDetailID.Value = null;

            rbAddProcedureEntry.Text = "Add Procedure";
            rbAddProcedureEntry.ToolTip = "Click to add procedure entry to the encounter";
        }
        rgProcedures.Visible = true;
        pnlAddProcedureBtn.Visible = true;

        pnlAddProcedureRegion.Visible = false;

        // clear values
        rtbDetailCode.Text = "";
        rtbDescription.Text = "";
        rtbDetailName.Text = "";

        fillReportGrid();
    }

    /// <summary>
    /// Cancel adding procedure
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void rbCancelAddProcedureEntry_Click(object sender, EventArgs e)
    {
        rgProcedures.Visible = true;
        pnlAddProcedureBtn.Visible = true;

        // clear values
        rtbDetailCode.Text = "";
        rtbDescription.Text = "";
        rtbDetailName.Text = "";
        hfEncounterDetailID.Value = null;

        rbAddProcedureEntry.Text = "Add Procedure";
        rbAddProcedureEntry.ToolTip = "Click to add procedure entry to the encounter";

        pnlAddProcedureRegion.Visible = false;
    }

    #endregion

    #region Private Methods

    /// <summary>
    /// Fills the report grid with entries from the db.
    /// </summary>
    private void fillReportGrid()
    {
        pnlAddProcedureBtn.Visible = IsEditable;

        rgProcedures.DataSource = EncounterDataManager.GetEncounterDetailsByEncounterAndType(VisitIdentifier, EncounterDetailType.CPT);
        rgProcedures.DataBind();
    }

    #endregion
}