﻿using System.Linq;
using System.Web.UI.WebControls;
using Telerik.Web.UI;
using ViewModels;
using ViewModels.Exam;

namespace Controls.ExamTabv2
{
    public partial class ExamTabMotilityUserControl : BaseEventItem<MotilityEventModel>
    {
        #region Implemented Members

        protected override void MapValidationGroup()
        {
            // Nothing to do here yet
        }

        protected override void MapTestSummary()
        {
            ucTestSummary.ViewModel = EventViewModel;
            ucTestSummary.TestPerformExpandValues = MotilityEventModel.ShowTestDetailValues;
        }

        protected override void MapTestDetails()
        {
            // Pursuit
            lblPursuit.Text = EventViewModel.Pursuit.DisplayText;
            ucPursuit.Question = EventViewModel.Pursuit;

            // Saccades
            lblSaccades.Text = EventViewModel.Saccades.DisplayText;
            ucSaccades.Question = EventViewModel.Saccades;

            // Nystagmus
            lblNystagmus.Text = EventViewModel.Nystagmus.DisplayText;
            ucNystagmus.Question = EventViewModel.Nystagmus;
            ucNystagmus.ExtendDdlOnItemDataBound = ucNystagmus_OnItemDataBound;

            ucNystagmusOptions.EmptyMessage = EventViewModel.NystagmusOptions.DisplayText;
            ucNystagmusOptions.Question = EventViewModel.NystagmusOptions;

            // Motility
            DataBindMotilityGrid();

            // Cranial Nerve
            ucCranialNerve.EmptyMessage = EventViewModel.CranialNerve.DisplayText;
            ucCranialNerve.Question = EventViewModel.CranialNerve;
        }

        protected override void PerformForcedRefresh()
        {
            // Force Refresh on child controls
            ucTestSummary.PerformForcedRefresh();
        }

        protected override MotilityEventModel SaveItem()
        {
            ucTestSummary.UpdateViewModel(EventViewModel);

            // Pursuit
            EventViewModel.Pursuit = ucPursuit.GetUpdatedQuestion();

            // Saccades
            EventViewModel.Saccades = ucSaccades.GetUpdatedQuestion();

            // Nystagmus
            EventViewModel.Nystagmus = ucNystagmus.GetUpdatedQuestion();
            EventViewModel.NystagmusOptions = ucNystagmusOptions.GetUpdatedQuestion();

            // Motility handled by gridview controls

            // Cranial Nerve
            EventViewModel.CranialNerve = ucCranialNerve.GetUpdatedQuestion();

            return EventViewModel;
        }

        #endregion

        #region Events

        protected void gvMotility_OnItemDataBound(object sender, GridItemEventArgs e)
        {
            if (e.Item.ItemType == GridItemType.EditItem)
            {
                var dataItem = e.Item.DataItem as Choice;

                if (dataItem != null && MotilityEventModel.MotilityNumberComment.Any(c => c == dataItem.Code))
                {
                    var regExValidator = (RegularExpressionValidator) e.Item.FindControl("regExValMotility");
                    regExValidator.Enabled = true;
                }
            }

            if (e.Item.ItemType == GridItemType.Footer)
            {
                RadComboBox motilityComboBox = (RadComboBox) e.Item.FindControl("ddlMotility");
                MapSelectField(motilityComboBox, EventViewModel.Motility);
            }
        }

        protected void gvMotility_OnItemCommand(object sender, GridCommandEventArgs e)
        {
            if (e.CommandName == "Edit")
            {
                gvMotility.EditIndexes.Add(e.Item.ItemIndex);
                gvMotility.MasterTableView.ShowFooter = false;
            }

            if (e.CommandName == "Delete")
            {
                var selectedChoice = e.Item.DataItem as Choice;

                if (selectedChoice != null)
                {
                    EventViewModel.Motility.SelectedChoices.RemoveAll(c => c.Code == selectedChoice.Code);
                }
            }

            if (e.CommandName == "Cancel")
            {
                gvMotility.EditIndexes.Clear();
                gvMotility.MasterTableView.ShowFooter = true;
            }

            if (e.CommandName == "Update")
            {
                var dataView = e.Item.DataItem as Choice;
                Choice selectedChoice =
                    dataView != null
                        ? EventViewModel.Motility.SelectedChoices.FirstOrDefault(c => c.Code == dataView.Code)
                        : null;

                if (selectedChoice != null)
                {
                    EventViewModel.Motility.SelectedChoices.Remove(selectedChoice);

                    TextBox tbMotilityComment = (TextBox)e.Item.FindControl("tbMotilityComment");
                    selectedChoice.Comment = FormatComment(selectedChoice.Code, tbMotilityComment.Text);

                    EventViewModel.Motility.SelectedChoices.Add(selectedChoice);
                }

                gvMotility.MasterTableView.ShowFooter = true;
            }

            if (e.CommandName == "Add")
            {
                RadComboBox comboBox = (RadComboBox)e.Item.FindControl("ddlMotility");
                string selectedChoiceCode = comboBox.SelectedValue;
                Choice selectedChoice =
                    EventViewModel.Motility.AllChoices.FirstOrDefault(c => c.Code == selectedChoiceCode);

                if (selectedChoice != null)
                {
                    TextBox tbMotilityComment = (TextBox)e.Item.FindControl("tbMotilityComment");
                    selectedChoice.Comment = FormatComment(selectedChoice.Code, tbMotilityComment.Text);

                    EventViewModel.Motility.SelectedChoices.Add(selectedChoice);

                    comboBox.Text = string.Empty;
                    tbMotilityComment.Text = string.Empty;
                }
            }

            DataBindMotilityGrid();
        }

        protected void ddlMotility_OnItemDataBound(object sender, RadComboBoxItemEventArgs e)
        {
            Choice choice = (Choice) e.Item.DataItem;
            e.Item.Attributes["RequiresComment"] = choice.RequiresComment.ToString().ToLower();
            e.Item.Attributes["RegExValMotilityEnabled"] =
                MotilityEventModel.MotilityNumberComment.Any(c => c == choice.Code).ToString().ToLower();
        }

        protected void ucNystagmus_OnItemDataBound(object sender, RadComboBoxItemEventArgs e)
        {
            Choice choice = (Choice) e.Item.DataItem;
            e.Item.Attributes["ExpandNystagmus"] =
                (choice.Code == MotilityEventModel.ChoiceCodes.Nystagmus.Present).ToString().ToLower();
        }

        #endregion

        #region Private Methods

        private void DataBindMotilityGrid()
        {
            gvMotility.MasterTableView.Columns[0].HeaderText = EventViewModel.Motility.DisplayText;
            gvMotility.DataSource = EventViewModel.Motility.SelectedChoicesOrderd;
            gvMotility.DataBind();
        }

        private string FormatComment(string choiceCode, string commentText)
        {
            if (MotilityEventModel.MotilityNumberComment.Any(c => c == choiceCode))
            {
                if (!commentText.EndsWith("PD"))
                {
                    commentText = commentText.Trim() + " PD";
                }
            }

            return commentText;
        }

        #endregion
    }
}