﻿using System.Linq;
using System.Web.UI.WebControls;
using Telerik.Web.UI;
using ViewModels;
using ViewModels.Exam;

namespace Controls.ExamTabv2
{
    /// <summary>
    /// The Exam Tab Prosthetic Devices User Control
    /// </summary>
    public partial class ExamTabProstheticDevicesUserControl : BaseEventItem<ProstheticDevicesEventModel>
    {
        #region Implemented Methods

        protected override void MapValidationGroup()
        {
            // Nothing to do here
        }

        protected override void MapTestSummary()
        {
            ucTestSummary.ViewModel = EventViewModel;
        }

        protected override void MapTestDetails()
        {
            ucOptions.Question = EventViewModel.Options;
            DataBindImplantTypeGridView();
        }

        /// <summary>Forces the refresh.</summary>
        protected override void PerformForcedRefresh()
        {
            // Force Refresh on child controls
            ucTestSummary.PerformForcedRefresh();
        }

        /// <summary>Saves the item.</summary>
        /// <returns>Updated Lens Event Model</returns>
        protected override ProstheticDevicesEventModel SaveItem()
        {
            ucTestSummary.UpdateViewModel(EventViewModel);
            EventViewModel.Options = ucOptions.GetUpdatedQuestion();

            return EventViewModel;
        }

        #endregion

        protected void gvImplantTypes_OnItemDataBound(object sender, GridItemEventArgs e)
        {
            if (e.Item.ItemType == GridItemType.EditItem)
            {
                var dataItem = e.Item.DataItem as Choice;

                if (dataItem != null && dataItem.Code == ProstheticDevicesEventModel.ChoiceCodes.ImplantTypes.Size)
                {
                    var compValidator = (RequiredFieldValidator)e.Item.FindControl("compValImplantTypes");
                    compValidator.Enabled = true;
                }
            }

            if (e.Item.ItemType == GridItemType.Footer)
            {
                RadComboBox implantTypeComboBox = (RadComboBox)e.Item.FindControl("ddlImplantType");
                MapSelectField(implantTypeComboBox, EventViewModel.ImplantTypes);
            }
        }

        protected void gvImplantTypes_OnItemCommand(object sender, GridCommandEventArgs e)
        {
            if (e.CommandName == "Edit")
            {
                gvImplantTypes.EditIndexes.Add(e.Item.ItemIndex);
                gvImplantTypes.MasterTableView.ShowFooter = false;
            }

            if (e.CommandName == "Delete")
            {
                var selectedChoice = e.Item.DataItem as Choice;

                if (selectedChoice != null)
                {
                    EventViewModel.ImplantTypes.SelectedChoices.RemoveAll(c => c.Code == selectedChoice.Code);
                }
            }

            if (e.CommandName == "Cancel")
            {
                gvImplantTypes.EditIndexes.Clear();
                gvImplantTypes.MasterTableView.ShowFooter = true;
            }

            if (e.CommandName == "Update")
            {
                var dataView = e.Item.DataItem as Choice;
                Choice selectedChoice =
                    dataView != null
                        ? EventViewModel.ImplantTypes.SelectedChoices.FirstOrDefault(c => c.Code == dataView.Code)
                        : null;

                if (selectedChoice != null)
                {
                    EventViewModel.ImplantTypes.SelectedChoices.Remove(selectedChoice);

                    TextBox tbImplantTypeCmt = (TextBox)e.Item.FindControl("tbImplantTypeCmt");
                    selectedChoice.Comment = tbImplantTypeCmt.Text;

                    EventViewModel.ImplantTypes.SelectedChoices.Add(selectedChoice);
                }

                gvImplantTypes.MasterTableView.ShowFooter = true;
            }

            if (e.CommandName == "Add")
            {
                RadComboBox comboBox = (RadComboBox)e.Item.FindControl("ddlImplantType");
                string selectedChoiceCode = comboBox.SelectedValue;
                Choice selectedChoice =
                    EventViewModel.ImplantTypes.AllChoices.FirstOrDefault(c => c.Code == selectedChoiceCode);

                if (selectedChoice != null)
                {
                    TextBox tbImplantTypeCmt = (TextBox)e.Item.FindControl("tbImplantTypeCmt");
                    selectedChoice.Comment = tbImplantTypeCmt.Text;

                    EventViewModel.ImplantTypes.SelectedChoices.Add(selectedChoice);

                    comboBox.Text = string.Empty;
                    tbImplantTypeCmt.Text = string.Empty;
                }
            }

            DataBindImplantTypeGridView();
        }

        protected void ddlImplantTypes_OnItemDataBound(object sender, RadComboBoxItemEventArgs e)
        {
            Choice choice = (Choice)e.Item.DataItem;
            e.Item.Attributes["RequiresComment"] = choice.RequiresComment.ToString().ToLower();
            e.Item.Attributes["CompValImplantTypeEnabled"] =
                (choice.Code == ProstheticDevicesEventModel.ChoiceCodes.ImplantTypes.Size).ToString().ToLower();
        }

        private void DataBindImplantTypeGridView()
        {
            gvImplantTypes.MasterTableView.Columns[0].HeaderText = EventViewModel.ImplantTypes.DisplayText;
            gvImplantTypes.DataSource = EventViewModel.ImplantTypes.SelectedChoicesOrderd;
            gvImplantTypes.DataBind();
        }
    }
}